/**
 * @file
 * VuoList_VuoOscMessage C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoOscMessage_H
#define VuoList_VuoOscMessage_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoOscMessage VuoList_VuoOscMessage
 * A list of @ref VuoOscMessage elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoOscMessage_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoOscMessage elements.
 */
typedef const struct VuoList_VuoOscMessage_struct { void *l; } * VuoList_VuoOscMessage;
/// @}
#endif

/**
 * Creates a new list of @ref VuoOscMessage elements.
 */
VuoList_VuoOscMessage VuoListCreate_VuoOscMessage(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoOscMessage VuoListCopy_VuoOscMessage(const VuoList_VuoOscMessage list);

/**
 * Returns the @ref VuoOscMessage at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoOscMessage VuoListGetValue_VuoOscMessage(const VuoList_VuoOscMessage list, const unsigned long index);

/**
 * Changes the @ref VuoOscMessage at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoOscMessage(const VuoList_VuoOscMessage list, const VuoOscMessage value, const unsigned long index);

/**
 * Inserts the `VuoOscMessage` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoOscMessage(const VuoList_VuoOscMessage list, const VuoOscMessage value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoOscMessage(VuoList_VuoOscMessage list, const VuoOscMessage value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoOscMessage(VuoList_VuoOscMessage list, const VuoOscMessage value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoOscMessage(VuoList_VuoOscMessage list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoOscMessage(VuoList_VuoOscMessage list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoOscMessage(VuoList_VuoOscMessage list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoOscMessage(VuoList_VuoOscMessage list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoOscMessage(VuoList_VuoOscMessage list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoOscMessage(VuoList_VuoOscMessage list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoOscMessage(VuoList_VuoOscMessage list);

/**
 * Removes the `VuoOscMessage` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoOscMessage(VuoList_VuoOscMessage list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoOscMessage(const VuoList_VuoOscMessage list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoOscMessage VuoList_VuoOscMessage_valueFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoOscMessage_jsonFromValue(const VuoList_VuoOscMessage value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoOscMessage_summaryFromValue(const VuoList_VuoOscMessage value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoOscMessage VuoList_VuoOscMessage_valueFromString(const char *str);
char * VuoList_VuoOscMessage_stringFromValue(const VuoList_VuoOscMessage value);
/// @}

/**
 * @}
 */

#endif
