/**
 * @file
 * VuoList_VuoSceneObject C type definition.
 *
 * @copyright Copyright © 2012–2014 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoSceneObject_H
#define VuoList_VuoSceneObject_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoSceneObject VuoList_VuoSceneObject
 * A list of @ref VuoSceneObject elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoSceneObject_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoSceneObject elements.
 */
typedef const struct VuoList_VuoSceneObject_struct { void *l; } * VuoList_VuoSceneObject;
/// @}
#endif

/**
 * Creates a new list of @ref VuoSceneObject elements.
 */
VuoList_VuoSceneObject VuoListCreate_VuoSceneObject(void);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoSceneObject VuoListCopy_VuoSceneObject(const VuoList_VuoSceneObject list);

/**
 * Returns the @ref VuoSceneObject at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoSceneObject VuoListGetValue_VuoSceneObject(const VuoList_VuoSceneObject list, const unsigned long index);

/**
 * Changes the @ref VuoSceneObject at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoSceneObject(const VuoList_VuoSceneObject list, const VuoSceneObject value, const unsigned long index);

/**
 * Inserts the `VuoSceneObject` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoSceneObject(const VuoList_VuoSceneObject list, const VuoSceneObject value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoSceneObject(VuoList_VuoSceneObject list, const VuoSceneObject value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoSceneObject(VuoList_VuoSceneObject list, const VuoSceneObject value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoSceneObject(VuoList_VuoSceneObject list, const unsigned long indexA, const unsigned long indexB);

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoSceneObject(VuoList_VuoSceneObject list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoSceneObject(VuoList_VuoSceneObject list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoSceneObject(VuoList_VuoSceneObject list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoSceneObject(VuoList_VuoSceneObject list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoSceneObject(VuoList_VuoSceneObject list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoSceneObject(VuoList_VuoSceneObject list);

/**
 * Removes the `VuoSceneObject` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoSceneObject(VuoList_VuoSceneObject list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoSceneObject(const VuoList_VuoSceneObject list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoSceneObject VuoList_VuoSceneObject_valueFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoSceneObject_jsonFromValue(const VuoList_VuoSceneObject value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoSceneObject_summaryFromValue(const VuoList_VuoSceneObject value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoSceneObject VuoList_VuoSceneObject_valueFromString(const char *str);
char * VuoList_VuoSceneObject_stringFromValue(const VuoList_VuoSceneObject value);
/// @}

/**
 * @}
 */

#endif
